#include <windows.h>
#include <stdio.h>

// Enable/disable privilege routine
BOOL SetPrivilege(
				  HANDLE hToken,	// access token handle
				  LPCTSTR lpszPrivilege,    // name of privilege to enable/disable
				  BOOL bEnablePrivilege    // to enable (or disable privilege)
				  )
{
	// Token privilege structure
	TOKEN_PRIVILEGES tp;
	// Used by local system to identify the privilege
	LUID luid;
	
	if(!LookupPrivilegeValue(
		NULL,				// lookup privilege on local system
		lpszPrivilege,		// privilege to lookup
		&luid))				// receives LUID of privilege
	{
		wprintf(L"LookupPrivilegeValue() failed, error: %u\n", GetLastError());
		return FALSE;
	}
	else
		wprintf(L"LookupPrivilegeValue() - \"%s\" found!\n", lpszPrivilege);
	
	// Number of privilege
	tp.PrivilegeCount = 1;
	// Assign luid to the 1st count
	tp.Privileges[0].Luid = luid;
	
	// Enable/disable
	if(bEnablePrivilege)
	{
		// Enable
		tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
		wprintf(L"\"%s\" was enabled!\n", lpszPrivilege);
	}
	else
	{
		// Disable
		tp.Privileges[0].Attributes = 0;
		wprintf(L"\"%s\" was disabled!\n", lpszPrivilege);
	}
	
	// Adjusting the new privilege
	if(!AdjustTokenPrivileges(
		hToken,
		FALSE,	// If TRUE, function disables all privileges,
				// if FALSE the function modifies privilege based on the tp
		&tp,
		sizeof(TOKEN_PRIVILEGES),
		(PTOKEN_PRIVILEGES) NULL,
		(PDWORD) NULL))
	{
		wprintf(L"AdjustTokenPrivileges() failed to adjust the new privilege, error: %u\n", GetLastError());
		return FALSE;
	}
	else
	{
		printf("AdjustTokenPrivileges() is OK - new privilege was adjusted!\n");
	}
	return TRUE;
}

int wmain(int argc, WCHAR **argv)
{
	// The privilege to be adjusted
	LPCTSTR lpszPrivilege = L"SeSecurityPrivilege";
	// Change this BOOL value to set/unset the SE_PRIVILEGE_ENABLED attribute
	// Initially to enable
	BOOL bEnablePrivilege = TRUE;
	HANDLE hToken;
	BOOL bRetVal;
	
	// Open a handle to the access token for the calling process
	if(!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES, &hToken))
	{
		wprintf(L"OpenProcessToken() failed, error %u\n", GetLastError());
		return FALSE;
	}
	else
		wprintf(L"OpenProcessToken() is OK, got the handle!\n");
	
	// Call the user defined SetPrivilege() function to
	// enable and set the needed privilege
	bRetVal = SetPrivilege(hToken, lpszPrivilege, bEnablePrivilege);
	
	if(!bRetVal)
	{
		wprintf(L"Failed to enable privilege, error %u\n", GetLastError());
		return FALSE;
	}
	else
		wprintf(L"The privilege was enabled!\n");
	
	//**************************************************
	// TODO: Complete your task which need the privilege
	wprintf(L"\nI am completing my task that need a privilege...\n\n");
	//**************************************************
	
	// After we have completed our task, don't forget to disable the privilege
	bEnablePrivilege = FALSE;
	bRetVal = SetPrivilege(hToken, lpszPrivilege, bEnablePrivilege);
	if(!bRetVal)
	{
		wprintf(L"Failed to disable the privilege, error %u\n", GetLastError());
		return FALSE;
	}
	else
		wprintf(L"The privilege was disabled!\n");
	
	return 0;
}
